%
% This function creates the backprojected rays defined by the corresponding
% projection matrices and 2D points.
%
% @param P A 1xn cell array. The set of projection matrices encoding 
%          the camera's internal and external parameters.
%
% @param points2D A 2xn matrix, representing a 2D point in each of the images.
%                 n is the number of projection matrices.
% 
% @return raySet A 6xn matrix representing the set of backprojected rays [p;n].
%                The first three rows, p, are the coordinates of the 3D point 
%                from which the ray emenates and the remaining three rows,
%                n, are the ray's direction (||n||=1).
%
% @author Stefan Wiesner (stefan_wiesner@freenet.de)
%
function raySet = createBackprojectedRaySet(P, points2D)

n = size(P,2);

raySet = [];
for i=1:n
  P3inv = inv(P{i}(1:3,1:3));
  rayOrigin = -P3inv * P{i}(1:3,4);
  rayDirection = P3inv * [points2D(:,i);1];
  raySet = [raySet, [rayOrigin; rayDirection./norm(rayDirection)]];
end
