%
% This function returns the fundamental matrix representing the epipolar 
% geometry defined by the two cameras, P_l (left camera) and 
% P_r (right camera): 
%     F = crossMatrix(e_r)*P_r*pinv(P_l), 
%     where e_r = P_r*c and P_l*c = 0.
% 
%
% @param Pl Projection matrix for the left camera.
%
% @param Pr Projection matrix for the right camera.
%
% @return F The fundamental matrix representing the epipolar geometry such
%         that a point x in the left image is transformed to the epipolar
%         line Fx in the right image.
%                    
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function F=epipolarGeometry(Pl, Pr)

 F = crossMatrix(Pr*null(Pl))*Pr*pinv(Pl);
