function vaxdrargout = fiducialLocalization(varargin)
% FIDUCIALLOCALIZATION M-file for fiducialLocalization.fig
%      FIDUCIALLOCALIZATION, by itself, creates a new FIDUCIALLOCALIZATION or raises the existing
%      singleton*.
%
%      H = FIDUCIALLOCALIZATION returns the handle to a new FIDUCIALLOCALIZATION or the handle to
%      the existing singleton*.
%
%      FIDUCIALLOCALIZATION('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in FIDUCIALLOCALIZATION.M with the given input arguments.
%
%      FIDUCIALLOCALIZATION('Property','Value',...) creates a new FIDUCIALLOCALIZATION or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before fiducialLocalization_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to fiducialLocalization_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".c
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Copyright 2002-2003 The MathWorks, Inc.

% Edit the above text to modify the response to help fiducialLocalization

% Last Modified by GUIDE v2.5 15-May-2009 12:42:30

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @fiducialLocalization_OpeningFcn, ...
                   'gui_OutputFcn',  @fiducialLocalization_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% --------------------------------------------------------------------

% --- Executes just before fiducialLocalization is made visible.
% This function has no output args, see OutputFcn.
function fiducialLocalization_OpeningFcn(hObject, eventdata, handles, varargin)

    %choose default command line output for fiducialLocalization
  handles.output = hObject;

             %set this variable to 1 if the code should output data that is
             %only usefull for illustrating the method in published material
  handles.DATA_FOR_PUBLICATION = 0;
  
      %set the figure's colormap (each figure has its own) to gray scale
      %for viewing X-ray images 
  colormap(gray(256));

       %color used during "manual" fiducial localization
  handles.MANUAL_FIDUCIAL_COLOR = [0.5, 0, 1];
  
     %color table used to display the fiducials
  handles.FIDUCIAL_COLOR_TABLE = {[1, 0, 0],...
                                  [0, 1, 0],...
                                  [0, 0, 1],...
                                  [1, 1, 0],...
                                  [0, 1, 1],...
                                  [1, 0, 1],...
                                  [0, 0, 0],...
                                  [1, 1, 1]};
                                  

        %frame that we display after successful loading
  handles.START_FRAME = 1;      
         %do we show the projections of the localized fiducials
  handles.projectFiducials = 0;
  
         %parameters are already saved, this is an initialization value
  handles.fiducialParametersSaved = 1;

          %expected radius of spherical fiducials in image
  handles.EXPECTED_RADIUS_RANGE = [5.0, 25.0];
  %handles.EXPECTED_RADIUS_RANGE = [2.0, 12.0];
  
           %a circular fiducial is valid if its number of supporting pixels
           %is greater than MINIMAL_CIRCUMFERENCE_PERCENTAGE*2*pi*r, where
           %the radius r is estimated in pixels
  handles.MINIMAL_CIRCUMFERENCE_PERCENTAGE = 0.3;
    
         %when tracking the fiducials assuming they are close to the previous
         %location we search around that location using this window size
  handles.HALF_SEARCH_WINDOW = 55.0;
  
                  %the dicom info (contains frame size, number of frames etc.)
  handles.info = struct([]);     
          %index of currently displayed frame
  handles.currentDisplayFrameNumber = -1;
              %array containing the 3D fiducial coordinates
  handles.fiducialCoordinates = [];
           %a cell array containing cell arrays with the circular fiducial
           %parameters [x;y;r]
  handles.allFiducialParameters = cell(0);
  
           %the currently displayed frame
  handles.currentDisplayFrame = [];  
   
           %camera matrices corresponding to the images in the cine loop
  handles.cameraMatrices = cell(0);
  
  handles.zoomOn = 0;
  
  set(handles.figure1,'CloseRequestFcn',@closeGUI);
  
         %update handles structure
  guidata(hObject, handles);

% --------------------------------------------------------------------

function varargout = fiducialLocalization_OutputFcn(hObject, eventdata, handles) 

% Get default command line output from handles structure
varargout{1} = handles.output;


% --------------------------------------------------------------------

function loadCineLoop_Callback(hObject, eventdata, handles)

                %open the file dialog and choose the DICOM file
  fileFilterSpecification = {'*.dcm;*.ima','DICOM files (*.dcm, *.ima)';...
                             '*.*','All files (*.*)'};
  fileDialogTitle = 'Open cine loop';  
  [fileName, pathName] = uigetfile(fileFilterSpecification,...
                                   fileDialogTitle,...
                                   'MultiSelect','off');
            %user chose a file 
  if fileName~=0 
   try 
     info = dicominfo(strcat(pathName,fileName));
   catch
     errordlg(sprintf('File (%s) is not in DICOM format.',fileName),... 
              'Invalid File', 'modal');
     return;
   end
         %check if number of images in file matches the number of projection 
         %matrices
   if ~isempty(handles.cameraMatrices) 
    if size(handles.cameraMatrices,2) ~= info.NumberOfFrames
        errordlg('Number of projection matrices does not match the number of frames in the cine loop.',...
                 'Error', 'modal');
             return;
    end
   end
      
   handles.info = info;
   handles.currentDisplayFrameNumber = handles.START_FRAME;  
   
   handles.currentFiducialParameters = cell(1,handles.info.NumberOfFrames);
   
           %has the data been saved
   handles.fiducialParametersSaved = 0;           
   try 
     handles.currentDisplayFrame = readFrame(handles.info, handles.currentDisplayFrameNumber);
   catch
     errordlg(lasterr(), 'Failed reading frame', 'modal');
     return;
   end
      
   set(handles.imageAxes,'CLim', [min(handles.currentDisplayFrame(:)),...
                                  max(handles.currentDisplayFrame(:))] );
   
         %remove all data estimated using previous data set and initialize
         %the GUI with the updated data
   clearData(handles);   
   initializeGUIAfterLoadingImage(guidata(gcbo));
  end
 
% --------------------------------------------------------------------
%
%Initialize the GUI after loading the first frame from the cine loop:
%1. Make all controls visible and active.
%2. Make all menu items active.
%3. Display the initial frame that was loaded.
%
function initializeGUIAfterLoadingImage(handles)

            %initialize the slider for browsing the cine loop's frames    
  set(handles.frameNumberSlider, 'Min', 1);
  set(handles.frameNumberSlider, 'Max', handles.info.NumberOfFrames);
  set(handles.frameNumberSlider, 'Value', handles.currentDisplayFrameNumber);
  set(handles.frameNumberSlider, 'SliderStep', [1/double(handles.info.NumberOfFrames),10/double(handles.info.NumberOfFrames)]);
  set(handles.frameNumberText, 'String', sprintf('%d',handles.currentDisplayFrameNumber));
  
  set(handles.uipanel1, 'Visible', 'on');
  set(handles.frameNumberSlider, 'Visible', 'on');
             %initialize the viewing behaviour controls
  set(handles.uipanel6, 'Visible', 'on');
  set(handles.zoomToolButton, 'Visible', 'on');
  set(handles.panToolButton, 'Visible', 'on');
  set(handles.windowLevelButton, 'Visible', 'on');
  set(handles.resetButton, 'Visible', 'on');
    
  set(handles.saveDataMenuItem, 'Enable', 'on');
  set(handles.loadDataMenuItem, 'Enable', 'on');
  set(handles.saveCurrentFigureMenuItem, 'Enable', 'on');
  set(handles.loadProjectionMatricesMenuItem, 'Enable', 'on');
  set(handles.editMenuItem, 'Enable', 'on');  
    
  guidata(gcbo, handles);
        %draw the initial frame   
  displayData(guidata(gcbo));
  
         %select the current axis and scale it to fit the image, this is 
         %done here once as all images in the cine-loop have the same size
  axes(handles.imageAxes);
  axis image;
  
% --------------------------------------------------------------------
%
% Save the fiducial parameters cell array as a single matrix to a text file.
%
function saveDataMenuItem_Callback(hObject, eventdata, handles)

              %open the file dialog and get the file name to which we need
               %to save the data. if an existing file is selected the file
               %dialog automatically confirms that it should be overwritten
  fileFilterSpecification = {'*.txt','Text file (*.txt)';...
                             '*.*','All files (*.*)'};
  fileDialogTitle = 'Save File Name';  
  [fileName, pathName] = uiputfile(fileFilterSpecification,...
                                   fileDialogTitle);
             %user chose a file 
   if fileName~=0                                  
         %save to the specified file  
     try        
       fiducialCoordinates = handles.fiducialCoordinates;
       save(strcat(pathName,fileName), 'fiducialCoordinates', '-ASCII', '-DOUBLE','-TABS');
       handles.fiducialParametersSaved = 1;
       guidata(gcbo, handles);
     catch
       errordlg(lasterr(), 'Failed saving file', 'modal');
     end
   end
% --------------------------------------------------------------------
%
%This is the function that does the actual work, localizes the fiducial
%in 3D starting with a manual rectangular region of interest that
%encompases the fiducial. Consists of the three phases described in
%Z. Yaniv, "Fiducial localization in C-arm based cone-beam CT",
%Proc. SPIE 7261, 72610I (2009), DOI:10.1117/12.811633'.
%
function localizeFiducialMenuItem_Callback(hObject, eventdata, handles)

if isempty(handles.cameraMatrices)
  errordlg('Forgot to load projection matrices', 'Error', 'modal');
  return;
end

currentFiducialParameters = cell(1,handles.info.NumberOfFrames);
                  
MIN_ANGULAR_SAMPLING = 5; %equivalent to ~2 degrees rotation
HALF_WINDOW_SIZE_INCREMENT = 50;
MIN_AGREEING_RAYS = 4;

           %a point is on the circle if it is less than 'CIRCLE_AGREEMENT_TOLERANCE' pixels
           %away
CIRCLE_AGREEMENT_TOLERANCE = 1.0; 

           %the rays intersect at the given point if the distance from the
           %point to the ray is less than 'RAY_AGREEMENT_TOLERANCE' mm
RAY_AGREEMENT_TOLERANCE = 1.0;
          
    %use every 'IMAGE_SAMPLING' image in the final localization
IMAGE_SAMPLING = 10;

%
% PHASE 1: localize in user specified region of interest
%

         %user localizes the fiducial in one of the images        
initialFiducialParameters = markFiducial();
if isempty(initialFiducialParameters)
  warndlg('Failed initial localization, please try another image.');
  return;
end
currentFiducialParameters{handles.currentDisplayFrameNumber} = initialFiducialParameters;

            %switch to the hourglass pointer and force a redraw 
set(handles.figure1,'Pointer','watch');
drawnow();
%
% PHASE 2: localize in adjacent images using epipolar constraint
%          note: the only reason the code is long is because of code 
%                replication (repeated code for searching in previous and 
%                next frames).
%

initialImageIndex = handles.currentDisplayFrameNumber;
nextImageIndexes = [initialImageIndex+MIN_ANGULAR_SAMPLING,...
                    initialImageIndex+2*MIN_ANGULAR_SAMPLING,...
                    initialImageIndex+3*MIN_ANGULAR_SAMPLING,...
                    initialImageIndex+4*MIN_ANGULAR_SAMPLING];
                  
prevImageIndexes = [initialImageIndex-MIN_ANGULAR_SAMPLING,...
                   initialImageIndex-2*MIN_ANGULAR_SAMPLING,...
                   initialImageIndex-3*MIN_ANGULAR_SAMPLING,...
                   initialImageIndex-4*MIN_ANGULAR_SAMPLING];

halfWindowSize = [HALF_WINDOW_SIZE_INCREMENT,... 
                  2*HALF_WINDOW_SIZE_INCREMENT,... 
                  3*HALF_WINDOW_SIZE_INCREMENT,...
                  4*HALF_WINDOW_SIZE_INCREMENT];
                
validImageIndexes = find(nextImageIndexes<handles.info.NumberOfFrames);
nextImageIndexes = nextImageIndexes(validImageIndexes);
nextHalfWindowSizes = halfWindowSize(validImageIndexes);

validImageIndexes = find(prevImageIndexes>1);
prevImageIndexes = prevImageIndexes(validImageIndexes);
prevHalfWindowSizes = halfWindowSize(validImageIndexes);

 %the number of frames we work on
allFrames = 1+...
            size(prevImageIndexes,2) + size(nextImageIndexes,2) +...
            size([1:IMAGE_SAMPLING:handles.info.NumberOfFrames],2);
allIndex = 1;          
progressBarHandle = waitbar(1/allFrames,'Please wait','Name', 'Localizing...');
       %set the progress bar color to blue
hw=findobj(progressBarHandle,'Type','Patch');
set(hw,'EdgeColor',[0 0 1],'FaceColor',[0 0 1]);

          %search in the previous frames
prevFiducialParameters = initialFiducialParameters;
n = size(prevImageIndexes,2);
for i=1:n
  try
   if ~isempty(prevFiducialParameters)
         %limit values so that they are inside image
     minCoords = max([1;1],... 
                     [prevFiducialParameters(1)-HALF_WINDOW_SIZE_INCREMENT;
                     prevFiducialParameters(2)-HALF_WINDOW_SIZE_INCREMENT]);
     maxCoords = min(flipud(size(handles.currentDisplayFrame)'),...
                     [prevFiducialParameters(1)+HALF_WINDOW_SIZE_INCREMENT;
                      prevFiducialParameters(2)+HALF_WINDOW_SIZE_INCREMENT]);
   else  %lost the fiducial, use initial fiducial location with larger search window     
                    %limit values so that they are inside image
     minCoords = max([1;1],...
                     [initialFiducialParameters(1)-prevHalfWindowSizes(i);
                      initialFiducialParameters(2)-prevHalfWindowSizes(i)]);
     maxCoords = min(flipud(size(handles.currentDisplayFrame)'),...
                     [initialFiducialParameters(1)+prevHalfWindowSizes(i);
                      initialFiducialParameters(2)+prevHalfWindowSizes(i)]);
   end
   
   currentFrame = readFrame(handles.info, prevImageIndexes(i));

   F = epipolarGeometry(handles.cameraMatrices{initialImageIndex},...
                        handles.cameraMatrices{prevImageIndexes(i)}); 
                      
   curPointSet = getEdges2DInWindowUsingEpipolarConstraint(currentFrame, minCoords, maxCoords,... 
                                             F*[initialFiducialParameters(1:2);1], 2*initialFiducialParameters(3));   
   [curCircleParameters, curConsensusSetIndexes] = ransacSphereConstrainedRadius(curPointSet, [0.8*initialFiducialParameters(3),1.2*initialFiducialParameters(3)],...
                                                                                 CIRCLE_AGREEMENT_TOLERANCE);
     %update the progress bar
   allIndex = allIndex+1;
   waitbar(allIndex/allFrames, progressBarHandle);
   currentFiducialParameters{prevImageIndexes(i)} = [curCircleParameters];
          %the assignment is valid even if curCircleParameters is empty
          %this is checked at the begining of the loop and the known
          %initial location is used to define the search window
   prevFiducialParameters = curCircleParameters;
    catch
      errordlg(lasterr(), 'Failed reading frame', 'modal');
      return;
  end
end

    %search in the next frames
prevFiducialParameters = initialFiducialParameters;
n = size(nextImageIndexes,2);
for i=1:n
  try
   if ~isempty(prevFiducialParameters)
         %limit values so that they are inside image
     minCoords = max([1;1],... 
                     [prevFiducialParameters(1)-HALF_WINDOW_SIZE_INCREMENT;
                     prevFiducialParameters(2)-HALF_WINDOW_SIZE_INCREMENT]);
     maxCoords = min(flipud(size(handles.currentDisplayFrame)'),...
                     [prevFiducialParameters(1)+HALF_WINDOW_SIZE_INCREMENT;
                      prevFiducialParameters(2)+HALF_WINDOW_SIZE_INCREMENT]);
   else  %lost the fiducial, use initial fiducial location with larger search window
                    %limit values so that they are inside image
     minCoords = max([1;1],...
                     [initialFiducialParameters(1)-nextHalfWindowSizes(i);
                      initialFiducialParameters(2)-nextHalfWindowSizes(i)]);
     maxCoords = min(flipud(size(handles.currentDisplayFrame)'),...
                     [initialFiducialParameters(1)+nextHalfWindowSizes(i);
                      initialFiducialParameters(2)+nextHalfWindowSizes(i)]);
   end

   currentFrame = readFrame(handles.info, nextImageIndexes(i));
   
   F = epipolarGeometry(handles.cameraMatrices{initialImageIndex},...
                        handles.cameraMatrices{nextImageIndexes(i)});
                      
   curPointSet = getEdges2DInWindowUsingEpipolarConstraint(currentFrame, minCoords, maxCoords,... 
                                             F*[initialFiducialParameters(1:2);1], 2*initialFiducialParameters(3));   
   [curCircleParameters, curConsensusSetIndexes] = ransacSphereConstrainedRadius(curPointSet, [0.8*initialFiducialParameters(3),1.2*initialFiducialParameters(3)],...
                                                                                 CIRCLE_AGREEMENT_TOLERANCE);
     %update the progress bar
   allIndex = allIndex+1;
   waitbar(allIndex/allFrames, progressBarHandle);                                                                               
   currentFiducialParameters{nextImageIndexes(i)} = [curCircleParameters];
          %the assignment is valid even if curCircleParameters is empty
          %this is checked at the begining of the loop and the known
          %initial location is used to define the search window
   prevFiducialParameters = curCircleParameters;
    catch
      errordlg(lasterr(), '', 'modal');
      return;
  end
end
         %get all the localized circles and estimate the interesection
         %of the backprojected rays.        
P = {};
fiducialParameters = [];
n = size(prevImageIndexes,2);
for i=1:n
  if ~isempty(currentFiducialParameters{prevImageIndexes(i)})
    P{end+1} = handles.cameraMatrices{prevImageIndexes(i)};
    fiducialParameters = [fiducialParameters,...
                          currentFiducialParameters{prevImageIndexes(i)}];
  end
end
n = size(nextImageIndexes,2);
for i=1:n
  if ~isempty(currentFiducialParameters{nextImageIndexes(i)})
    P{end+1} = handles.cameraMatrices{nextImageIndexes(i)};
    fiducialParameters = [fiducialParameters,currentFiducialParameters{nextImageIndexes(i)}];
  end
end
if size(P,2)<MIN_AGREEING_RAYS
  warndlg('Number of detected circles (initial) is too small.');
  return;
end
  
raySet = createBackprojectedRaySet(P, fiducialParameters(1:2,:));
[initialPnt3D, initialRayIntersectionConsensusSetIndexes] = ransacRayIntersections(raySet, RAY_AGREEMENT_TOLERANCE)
fiducialMedianRadius = median(fiducialParameters(3,:));
           %if too few inliers then I don't trust the localization           
if size(initialRayIntersectionConsensusSetIndexes,2)<MIN_AGREEING_RAYS
  close(progressBarHandle);
  set(handles.figure1,'Pointer','arrow');
  warndlg('Number of rays agreeing with initial intersection is too small.');
  return;
end

%
% PHASE 3: re-project onto images and localize
%

            %project the fiducial onto the images, redectect and
            %re-estimate
for i=1:IMAGE_SAMPLING:handles.info.NumberOfFrames
  projectedPnt = handles.cameraMatrices{i}*[initialPnt3D;1];
  projectedPnt = projectedPnt./projectedPnt(3);    
         %localize only inside image
  minY = round(projectedPnt(2) - handles.EXPECTED_RADIUS_RANGE(2));
  maxY = round(projectedPnt(2) + handles.EXPECTED_RADIUS_RANGE(2));
  minX = round(projectedPnt(1) - handles.EXPECTED_RADIUS_RANGE(2));
  maxX = round(projectedPnt(1) + handles.EXPECTED_RADIUS_RANGE(2));
  [yBounds, xBounds] = size(handles.currentDisplayFrame);  
  if minY>=1 && minY<= yBounds &&...
     maxY>=1 && maxY<= yBounds &&... 
     minX>=1 && minX<= xBounds &&...
     maxX>=1 && maxX<= xBounds 
   
    currentFrame = readFrame(handles.info, i);

    fiducialParameters = localize2DFiducialsUsingRansac(...
                         currentFrame(minY:maxY, minX:maxX),... 
                         [0.5*fiducialMedianRadius,1.5*fiducialMedianRadius],... 
                         handles.MINIMAL_CIRCUMFERENCE_PERCENTAGE);   
    if ~isempty(fiducialParameters)                       
      currentFiducialParameters{i} = [fiducialParameters(1)+minX-1;...
                                      fiducialParameters(2)+minY-1;
                                      fiducialParameters(3)];
    end
  end
          %update the progress bar               
  allIndex = allIndex+1;
  waitbar(allIndex/allFrames, progressBarHandle);                       
end


P ={};
points2D = [];
for i=1:IMAGE_SAMPLING:handles.info.NumberOfFrames
  if ~isempty(currentFiducialParameters{i})
    P{end+1} = handles.cameraMatrices{i};
    points2D = [points2D,currentFiducialParameters{i}(1:2,1)];
  end
end

    %matlab indexes start at one and the camera calibration matrices were
    %estimated with a coordinate system that starts at zero
points2D = [points2D(1,:)+1 ; points2D(2,:)-1 ];

raySet = createBackprojectedRaySet(P, points2D);
finalPnt3D = ransacRayIntersections(raySet, RAY_AGREEMENT_TOLERANCE)

       %this portion of the code is relevant only for generating data for
       %the paper, value is set in OpeningFcn     
if handles.DATA_FOR_PUBLICATION
          
  DEGREES2RADIANS = pi/180;
         %this parameter is acquisition specific 
  SWEEP_IN_DEGREES = 209;
  posesUsed = [];
  for i=1:IMAGE_SAMPLING:handles.info.NumberOfFrames
    if ~isempty(currentFiducialParameters{i})
      posesUsed = [posesUsed, i-1];
    end
  end
  
          %manually plot the x and y axis and set the text to degrees
  figureHandle = figure;          
  plot([0,0],[-1,1],'-k');
  hold on;
  plot([-1,1],[0,0],'-k');
  text(1.05,0,'0^o');
  text(0,1.05,'90^o');
  text(-1.15,0,'180^o');
  text(0,-1.05,'270^o');

           %plot the C-arm rotation using the default color (blue) and line 
           %style (solid), then plot the C-arm poses used in fiducial localization as red
           %x's
  rotationAngles = DEGREES2RADIANS*[0:0.2:SWEEP_IN_DEGREES];
  posesUsed = SWEEP_IN_DEGREES/(handles.info.NumberOfFrames-1)*DEGREES2RADIANS*posesUsed;
  trajectoryX = cos(rotationAngles);
  trajectoryY = sin(rotationAngles);
  plot(trajectoryX,trajectoryY);
  plot(cos(posesUsed),sin(posesUsed),'r*');
  axis equal;
  axis off;  
end %handles.DATA_FOR_PUBLICATION


handles.allFiducialParameters{end+1} = currentFiducialParameters;
handles.fiducialCoordinates = [handles.fiducialCoordinates, finalPnt3D];
   %update the progress bar and close it
waitbar(1, progressBarHandle);
close(progressBarHandle);

     %save the new fiducial data and update the flag that indicates that
     %the current fiducials haven't been saved to file
guidata(gcbo, handles);
handles.fiducialParametersSaved = 0;

set(handles.figure1,'Pointer','arrow');
displayData(guidata(gcbo));

% --------------------------------------------------------------------

function markFiducialMenuItem_Callback(hObject, eventdata, handles)

  handles.currentFiducialParameters{handles.currentDisplayFrameNumber} =... 
    markFiducial();
  guidata(gcbo, handles);
     %update the display with the newly detected fiducial
  displayData(guidata(gcbo));

  
% --------------------------------------------------------------------
%
% User indicates a rectangular region of interest that contains the circular
% fiducial. This function invokes the localizeFiducialsUsingRansac() which
% actually does the work.
% 
function fiducialParameters = markFiducial()
  fiducialParameters = [];
  
             %get the handle of the object whose callback is
             %currently executing    
  handles = guidata(gcbo);
  zoom(handles.figure1, 'off');
  pan(handles.figure1, 'off');
  
         %get user's ROI
  ok = ~waitforbuttonpress();    
  if ok
    point1 = get(handles.imageAxes,'CurrentPoint');    
    finalRect = rbbox;                   
    point2 = get(handles.imageAxes,'CurrentPoint');    
               % extract x and y
    point1 = point1(1,1:2);              
    point2 = point2(1,1:2);
    minCoords = min(point1, point2);
    maxCoords = max(point1, point2);
                  %need to flip for comparison as minCoords,maxCoords are
              %[x,y] and image size is given as [y,x]
    imageSizes = fliplr(size(handles.currentDisplayFrame));

            %limit the coordinates to be inside the image and the ROI must
            %be a valid rectangle, not one in which one or both sides have
            %dimension zero
    if all(maxCoords<=imageSizes) && all(maxCoords>=[1 1]) &&... 
       all(minCoords<=imageSizes) && all(minCoords>=[1 1]) &&...
       all(minCoords ~=maxCoords)

            %localize fiducials found in sub-image 
      set(handles.figure1,'Pointer','watch');
      fiducialParameters = localize2DFiducialsUsingRansac(handles.currentDisplayFrame(round(minCoords(2)):round(maxCoords(2)),...
                                                                                      round(minCoords(1)):round(maxCoords(1))),...
                                                          handles.EXPECTED_RADIUS_RANGE, handles.MINIMAL_CIRCUMFERENCE_PERCENTAGE);
      set(handles.figure1,'Pointer','arrow');
           %fiducial coordinates are in the sub-image coordinate system, save them in the original coordinate
           %system
      if ~isempty(fiducialParameters)
                    %modify the coordinate system to the original image
        fiducialParameters = [fiducialParameters(1,:) + round(minCoords(1))-1;... 
                              fiducialParameters(2,:) + round(minCoords(2))-1;... 
                              fiducialParameters(3,:)];
      end
    end
  end

% --------------------------------------------------------------------

function quitMenuItem_Callback(hObject, eventdata, handles)

quit(handles);

% --------------------------------------------------------------------

function closeGUI(src, evnt)
 
quit(guidata(gcbo));


% --------------------------------------------------------------------  

function quit(handles)

  if ~handles.fiducialParametersSaved &... 
     ~isempty(handles.fiducialCoordinates)    
    selectionStr = questdlg('Exit without saving?',...
                            'Discard Data Confirmation','Yes','No','No');
    if strcmp(selectionStr,'No')
      return;      
    end
  end
       %use delete instead of close as otherwise the closeGUI callback
       %function will be called recursively (the 'close' function generate the
       %close event which triggers the closeGUI function which would have
       %called close...).
  delete(handles.figure1);

% --------------------------------------------------------------------  

function displayData(handles)
       %select the axes on which to draw
  axes(handles.imageAxes);
       %clear the axes, otherwise the data is added to the axes object
       %on top of the previous data, the program keeps allocating more and
       %more memory which we don't need
  cla(handles.imageAxes);
      %draw the image with gray level scaling and remove the axes
      %(imagesc makes the axes visible so I need to remove it after 
      %every call to imagesc)               
  imagesc(handles.currentDisplayFrame, get(handles.imageAxes,'CLim'));
  set(handles.imageAxes, 'Visible', 'off');

  hold on;
             %draw the fiducials on the image
             
         %draw the fiducials that are "manually" marked on this image, still haven't estimate
         %this fiducials 3D location
  currentFiducials = handles.currentFiducialParameters{handles.currentDisplayFrameNumber};
  if ~isempty(currentFiducials)
    drawCircle(currentFiducials(1,1), currentFiducials(2,1), currentFiducials(3,1),...
               handles.imageAxes, '-', handles.MANUAL_FIDUCIAL_COLOR);
  end
  
       %draw 2D fiducials whose 3D location was already estimated
  n = size(handles.allFiducialParameters,2);
  for i=1:n
    circleParameters = handles.allFiducialParameters{i}{handles.currentDisplayFrameNumber};
    if~isempty(circleParameters)
      colorTableIndex = mod(i, size(handles.FIDUCIAL_COLOR_TABLE,2));
      if colorTableIndex == 0
        colorTableIndex = size(handles.FIDUCIAL_COLOR_TABLE,2);
      end
      drawCircle(circleParameters(1), circleParameters(2), circleParameters(3),...
                 handles.imageAxes, '-', handles.FIDUCIAL_COLOR_TABLE{colorTableIndex});
    end
  end
  
    %draw the projections of the localized fiducials
 if handles.projectFiducials
   if isempty(handles.cameraMatrices)
     errordlg('Forgot to load projection matrices, turn "Project Fiducials" off or load matrices', 'Error', 'modal');
     return;
   end
   if ~isempty(handles.fiducialCoordinates)
     P = handles.cameraMatrices{handles.currentDisplayFrameNumber};
     pnt2DHomogenous = P*[handles.fiducialCoordinates;ones(1,size(handles.fiducialCoordinates,2))];
     tmp = pnt2DHomogenous(3,:);
     projections = pnt2DHomogenous./tmp(ones(3,1),:);
          %matlab coordinates start at 1,1 and not 0,0
     plot(projections(1,:)+1, projections(2,:)+1, 'wx');
   end
 end
  
% --------------------------------------------------------------------

function frameNumberSlider_Callback(hObject, eventdata, handles)
          %get the new frame number
  frameNumber = round(get(hObject,'Value'));
  set(handles.frameNumberText,'String',sprintf('%d',frameNumber));
  handles.currentDisplayFrameNumber = frameNumber;
          %load the image and update display
  try          
    handles.currentDisplayFrame = readFrame(handles.info, handles.currentDisplayFrameNumber);
  catch
    errordlg(lasterr(), 'Failed reading frame', 'modal');
  end 
       %save the data and display
  guidata(gcbo, handles);     
  displayData(guidata(gcbo));
% --------------------------------------------------------------------
% --- Executes during object creation, after setting all properties.
function frameNumberSlider_CreateFcn(hObject, eventdata, handles)
% hObject    handle to frameNumberSlider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

% --------------------------------------------------------------------

function loadDataMenuItem_Callback(hObject, eventdata, handles)

               %open the file dialog and get the file name we want to read
  fileFilterSpecification = {'*.txt','Text file (*.txt)';...
                             '*.*','All files (*.*)'};
   fileDialogTitle = 'Fiducial Data File Name';  
   [fileName, pathName] = uigetfile(fileFilterSpecification,...
                                    fileDialogTitle,...
                                    'MultiSelect','off');
             %user chose a file 
   if fileName~=0 
      try 
        fiducialData = load(strcat(pathName,fileName));
        if size(fiducialData,1) == 3
            clearData(handles);
            handles.fiducialCoordinates = fiducialData;
            guidata(gcbo, handles);
            displayData(guidata(gcbo));
        else
            errordlg(message, 'Error in fiducial data', 'modal');
        end
      catch
        errordlg(lasterr(), 'Failed reading fiducial data file', 'modal');
      end
   end   
   
% --------------------------------------------------------------------
 
function loadProjectionMatricesMenuItem_Callback(hObject, eventdata, handles)

               %open the file dialog and get the file name to which we need
               %to save the data. if an existing file is selected the file
               %dialog automatically confirms that it should be overwritten
   fileFilterSpecification = {'*.txt','Text files (*.txt)';...
                              '*.*','All files (*.*)'};
   fileDialogTitle = 'Projection Matrices Data File Name';  
   [fileName, pathName] = uigetfile(fileFilterSpecification,...
                                    fileDialogTitle,...
                                    'MultiSelect','off');
             %user chose a file 
  if fileName~=0 
     try
       projectionMatrices = loadProjectionMatrices(strcat(pathName,fileName));
               %check that the number of projection matrices is the same as
               %the number of frames in the cine loop
       if size(projectionMatrices,2) ~= handles.info.NumberOfFrames
           errordlg('Number of projection matrices does not match the number of frames in the cine loop.',...
                    'Error', 'modal');
           return;     
       end
       handles.cameraMatrices = projectionMatrices;
       guidata(gcbo, handles);
     catch
       errordlg(lasterr(), 'Failed reading projection matrices data file', 'modal');
     end
  end

% --------------------------------------------------------------------

function saveCurrentFigureMenuItem_Callback(hObject, eventdata, handles)

              %open the file dialog and get the file name prefix
              %if any of the files exist ask for confirmation
  fileFilterSpecification = {'*.eps','Encapsulated Postscript (*.eps)';...
                             '*.emf','Enhanced Metafile (*.emf)';...
                             '*.fig','Matlab figure file (*.fig)';...
                             '*.*','All files (*.*)'};
  fileDialogTitle = 'Save File Name';  
  [fileName, pathName, filterIndex] = uiputfile(fileFilterSpecification,...
                                                fileDialogTitle);                                                                      
                                                                   
           %user chose a file 
  if fileName~=0
    indexes = strfind(fileName, '.');
    if size(indexes,2) == 0
      fileNamePrefix = fileName;
    else
      fileNamePrefix = fileName(1:indexes(end)-1);
    end            %save in all formats
    if filterIndex == 4
      savePlot(strcat(pathName, fileNamePrefix), handles.imageAxes);
    else        %save in specific format
      savePlot(strcat(pathName, fileNamePrefix),... 
               handles.imageAxes,... 
               fileFilterSpecification{filterIndex}(3:end));
    end
  end
  
% --------------------------------------------------------------------

%
%Executes on button press in resetButton.
%
function resetButton_Callback(hObject, eventdata, handles)
  zoom(handles.figure1,'off');
  zoom(handles.figure1, 'out');
  handles.zoomOn = 0;    
  pan(handles.figure1,'off');
  set(handles.imageAxes,'CLim', [min(handles.currentDisplayFrame(:)),...
                                max(handles.currentDisplayFrame(:))] );
  guidata(hObject, handles);
  
% --------------------------------------------------------------------

%
%Executes on button press in zoomToolButton.
%
function zoomToolButton_Callback(hObject, eventdata, handles)
  if handles.zoomOn
    zoom(handles.figure1,'off');
    handles.zoomOn = 0;    
  else
    zoom(handles.figure1,'on');
    handles.zoomOn = 1;    
  end
  guidata(hObject, handles);

% --------------------------------------------------------------------

%
%Executes on button press in panToolButton.
%
function panToolButton_Callback(hObject, eventdata, handles)

pan(handles.figure1);

% --------------------------------------------------------------------

function removeAllMenuItem_Callback(hObject, eventdata, handles)

clearData(handles);
     %get the updated handles structure for display
     %the handles structure is passed by value to the callback function, thus
     %the changes done in clearData() are not be reflected in the local copy
displayData(guidata(gcbo));

% --------------------------------------------------------------------

%
% Clear all the data, includes the 2D fiducial data 
% (currentFiducialParameters, allFiducialParameters), and 3D data
% (fiducialCoordinates).
%
function clearData(handles)

n = handles.info.NumberOfFrames; 
for i=1:n
  handles.currentFiducialParameters{i} = [];
end
handles.fiducialCoordinates = [];
handles.allFiducialParameters = cell(0);
guidata(gcbo, handles);

% --------------------------------------------------------------------

function removeLastMenuItem_Callback(hObject, eventdata, handles)

  handles.allFiducialParameters = {handles.allFiducialParameters{1:end-1}};
  handles.fiducialCoordinates = handles.fiducialCoordinates(:,1:end-1);
  guidata(gcbo, handles);
  displayData(guidata(gcbo));

% --------------------------------------------------------------------

%
%Executes on button press in windowLevelButton.
%
function windowLevelButton_Callback(hObject, eventdata, handles)

imcontrast(handles.imageAxes);

% --------------------------------------------------------------------

function projectFiducialsMenuItem_Callback(hObject, eventdata, handles)

  handles.projectFiducials = ~handles.projectFiducials;
  if handles.projectFiducials
    set(gcbo, 'Checked', 'on');
  else
    set(gcbo, 'Checked', 'off');
  end
  guidata(gcbo, handles);
  displayData(guidata(gcbo));
  
% --------------------------------------------------------------------

function localizeMarkedFiducialsMenuItem_Callback(hObject, eventdata, handles)

if isempty(handles.cameraMatrices)
  errordlg('Forgot to load projection matrices', 'Error', 'modal');
  return;
end

n = handles.info.NumberOfFrames; 
P ={};
points2D = [];
for i=1:n
  if ~isempty(handles.currentFiducialParameters{i})
    P{end+1} = handles.cameraMatrices{i};
    points2D = [points2D,handles.currentFiducialParameters{i}(1:2,1)];
  end
end

  %not enough fiducials were marked
if size(points2D,2)<2
  errordlg('Insufficient number of fiducials marked, at least two required.', 'Error', 'modal');
  return;
end

raySet = createBackprojectedRaySet(P, points2D);
rayAgreementTolerance = 1.0;
pnt3D = ransacRayIntersections(raySet, rayAgreementTolerance);
handles.fiducialCoordinates = [handles.fiducialCoordinates, pnt3D];
         %copy the 2D data to the cell array that is used for display
         %purposes and then clear the currentFiducialParameters so that the
         %same fiducial won't be added multiple times by accident (always
         %possible on purpose).
handles.allFiducialParameters{end+1} =  handles.currentFiducialParameters;

for i=1:n
  handles.currentFiducialParameters{i} = [];
end

guidata(gcbo, handles);
displayData(guidata(gcbo));

% --------------------------------------------------------------------
function aboutMenuItem_Callback(hObject, eventdata, handles)

str{1} = 'Software implementation of the work described in:';
str{2}=  '"Evaluation of Spherical Fiducial Localization in C-arm';
str{2} = strcat(str{2},' Cone-Beam CT using Patient Data", Z. Yaniv,');
str{2} = strcat(str{2},' Med. Phys., 37(10): 5298-5305, 2010.');

 %create a figure with the title 'Help', remove the standard 'Figure n:'
 %and no menu bar
screenSize = get( 0, 'ScreenSize' ); 
h = figure('Name', 'About',...
           'NumberTitle','off',...
           'MenuBar','none',... 
           'Position', [screenSize(3)/2.0,screenSize(4)/2.0,450,50],...
           'Color', [0.8,0.8,0.8]);
 %add a 100 line edit control to the figure and display the text
uicontrol('Parent',h,...
          'Units','normalized',...
          'hor', 'left',...
          'Position',[0.1,0.1,0.8,0.8],...
          'Style','text',...
          'Max',2,...          
          'String', str,...
          'BackgroundColor', [0.8,0.8,0.8]);

% --------------------------------------------------------------------
function helpContentsMenuItem_Callback(hObject, eventdata, handles)

try
  fileName='help.txt';
  str = textread(fileName,'%s','whitespace', '', 'delimiter','\n');

   %create a figure with the title 'Help', remove the standard 'Figure n:'
   %and no menu bar
  screenSize = get( 0, 'ScreenSize' );
  h = figure('Name', 'Help',...
             'NumberTitle', 'off',...
             'MenuBar', 'none',...
             'Position', [screenSize(3)/2.0,screenSize(4)/2.0,700,300],...
             'Color', [0.8,0.8,0.8]);
         
    %add a 100 line edit control to the figure and display the text
  uicontrol('Parent',h,...
            'Units','normalized',...
            'hor', 'left',...
            'Position',[0.1,0.1,0.8,0.8],...
            'Style','edit',...
            'Max',2,...
            'Enable','inactive',...
            'BackgroundColor', [0.8,0.8,0.8],...
            'String', str);
catch
  errordlg(sprintf('Help file (%s) not found.',fileName),...
           'Missing help file', 'modal');
end
            
% --------------------------------------------------------------------
function licenseMenuItem_Callback(hObject, eventdata, handles)

try 
  fileName='license.txt';
  str = textread(fileName,'%s','whitespace', '', 'delimiter','\n');

   %create a figure with the title 'Help', remove the standard 'Figure n:'
   %and no menu bar
  screenSize = get( 0, 'ScreenSize' ); 
  h = figure('Name', 'License',...
             'NumberTitle', 'off',...
             'MenuBar', 'none',...
             'Position', [screenSize(3)/2.0,screenSize(4)/2.0,700,300],...
             'Color', [0.8,0.8,0.8]);
         
    %add a 100 line edit control to the figure and display the text
  uicontrol('Parent',h,...
            'Units','normalized',...
            'hor', 'left',...
            'Position',[0.1,0.1,0.8,0.8],...
            'Style','edit',...
            'Max',2,...
            'Enable','inactive',...
            'BackgroundColor', [0.8,0.8,0.8],...
            'String', str);
catch
  errordlg(sprintf('License file (%s) not found.',fileName),...
           'Missing license file', 'modal');
end
  
% --------------------------------------------------------------------
function helpMenuItem_Callback(hObject, eventdata, handles)
%nothing to do when opening the help menu





