%
% This function returns the edge locations that are near the epipolar line
% in the given sub-image.
%
% @param img Perform edge detection on this image.
%
% @param minCoords Minimum coordinates of sub image we are interested in.
%
% @param maxCoords Maximum coordinates of sub image we are interested in.
%
% @param epipolarLine The line that we are interested in.
%
% @param maxDistanceFromLine Edges that we are interested in are inside the
%                            sub-image and are closer than this distance to
%                            the given epipolar line.
%                    
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function edgePoints = getEdges2DInWindowUsingEpipolarConstraint(img,... 
minCoords, maxCoords,epipolarLine, maxDistanceFromLine)

subImage = img(round(minCoords(2)):round(maxCoords(2)),...
               round(minCoords(1)):round(maxCoords(1)));

          %perform edge detection using the Canny detector, 
          %NOTE: the only reason we can use matlab's 'edge' function is
          %that the pixels are isotropic (see dicom header)
          %in general image processing in matlab does not have the notion
          %of pixel/voxel spacing, everything is assumed to be isotropic
std = 2.0;
[edgeImage,thresh] = edge(subImage, 'canny', [], std);
edgeIndexes = find(edgeImage == 1);

     %no edges found, no need to continue processing
if isempty(edgeIndexes)
  edgePoints = [];
  return;
end

[y,x] = ind2sub(size(edgeImage), edgeIndexes);


         %get the edge locations in the original image
x = x + round(minCoords(1))-1;
y = y + round(minCoords(2))-1;
edgePoints = [];
n = size(x,2);
distances = projective2DPointLineDistances([x';y'], epipolarLine); 
indexes = find(distances<maxDistanceFromLine);
edgePoints = [x(indexes)';y(indexes)'];
