%
% This function loads the projection matrices encoding the camera
% internal and external parameters throughout the image acquisition
% process.
%
% NOTE: It is assumed that the time interval between the camera poses
%       encoded in the projection matrices is uniform. That is, for all i,j 
%       timeInterval(P_i,P_{i+1}) == timeInterval(P_j,P_{j+1}).
% 
% @param projectionMatricesFileName The name of the ASCII file containing
%                                   the projection matrices. 
%
% @return P A cell array containing the projection matrices.
%
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function P = loadProjectionMatrices(projectionMatricesFileName)

 tmp = load(projectionMatricesFileName);
 if size(tmp,2) ~= 4 || mod(size(tmp,1),3) ~= 0
   error('Projection matrices file "%s" corrupted.');
 end
 n = size(tmp,1)/3;
 for i=1:n
   startIndex = (i-1)*3+1;
   P{i} = tmp(startIndex:startIndex+2,:);
 end
 
