%
% Compute the distance between the given set of 2D points and the 2D 
% projective line.
% 
% @param points A 2xn array of 2D points.
%
% @param line A 3x1 array with the projective line parameters.
%
%
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function distances = projective2DPointLineDistances(points, line) 

       %get two points on the projective line
if line(2) ~= 0.0
     %find the y value on the line for x=0.0
  intersectionPoints = [0.0;-line(3)/line(2)];  
     %find the y value on the line for x=1.0  
  intersectionPoints = [intersectionPoints,[1.0;(-line(3) - line(1))/line(2)]];
end

if line(1) ~= 0.0
     %find the x value on the line for y=0.0
  intersectionPoints = [intersectionPoints, [-line(3)/line(1); 0.0]];
     %find the x value on the line for y=1.0  
  intersectionPoints = [intersectionPoints,[(-line(3) - line(2))/line(1); 1.0]];
end

    %deal with degenerate cases where the line is at 45 degrees and we get
    %four intersections
intersectionPoints = unique(intersectionPoints', 'rows')';

         %compute the distances between all points and the line using
         %matlab's vectorial approach
         
p1 = intersectionPoints(:,1);
p2 = intersectionPoints(:,2);

p1s = p1(:,ones(size(points,2),1));
dir = p2-p1;
dirs = dir(:,ones(size(points,2),1));
tmp = points-p1s;
t = sum(tmp.*dirs,1);
t = t/(dir' * dir);
closestPoints = p1s + t(ones(1,2),:).*dirs;
distances = sqrt(sum((closestPoints - points).^2,1));
