%
% This function reads a single frame from a cine loop. 
% In practice it forwards the request to the relevant reading function. In
% our case the data format changes depending upon the way it was
% transferred, via DICOM transfer directly from the CBCT machine or via
% export to another machine (hospital's PACS) or device (writing to CD). In
% the latter cases the original images are resized and padded to a square
% 1024x1024 cine.
%
% @param info The dicom info structure of the file that should be read.
%
% @param i The frame number we want to read. Valid values are 
%          in [1,info.NumberOfFrames]
%
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function img = readFrame(info, i)

     %when we get the data from the siemens computer by burnning a CD or from the
     %hospital's PACS it is scaled and we need to undo this scaling so that the images are 
     %compatible with the calibration matrices, in all other cases I assume
     %that the data is uncompressed and matches the calibration matrices
  RESIZED_COLUMNS = 1024;
  RESIZED_ROWS = 1024;

  if info.Rows == RESIZED_ROWS && info.Columns == RESIZED_COLUMNS
    img = readFramePadded(info, i);
  else
    img = readFrameDicomTransferred(info, i);
  end
