%
% This function reads a single frame from a cine loop. I assume that the 
% pixel data is uncompressed and appears at the end of the DICOM header.
% This is valid for the current (Feb. 2008) data acquired with the
% Siemens Axiom Artis dFA system and transfered via DICOM transfer from
% the acquisition computer.
%
% @param info The dicom info structure of the file that should be read.
%
% @param i The frame number we want to read. Valid values are 
%          in [1,info.NumberOfFrames]
%
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function img = readFrameDicomTransferred(info, i)

  if i>info.NumberOfFrames || i<1
    error('Requested frame (%d) out of range of valid frames (1-%d).',...
          i, info.NumberOfFrames);
  end
  fileName = info.Filename;
  frameSizeInBytes = double(info.Rows)*double(info.Columns)*(double(info.BitsAllocated)/8);
  
                 %compute the location where pixel data starts, this is a
                 %hack
  fileInfo = dir(fileName);
  fileSize=fileInfo.bytes;
  offset = fileSize - frameSizeInBytes*info.NumberOfFrames;  
  fid = fopen(fileName);
  if fid == -1
    error('Failed to read frame (%d), file (%s) cannot be accessed.',...
          i, fileName);
  end  
           %skip from the begining of the file to the frame's start
           %location
  fseek(fid, offset + (i-1)*frameSizeInBytes, 'bof');
  img = fread(fid,[double(info.Columns), double(info.Rows)],'int16');
       %we need to rotate the image by 270^o so that it matches the
       %projection matrices
  img = rot90(img,3);
  fclose(fid);
